@@ -2,7 +2,6 @@
 
 use crate::ir::pcc::*;
 use crate::ir::types::*;
-use crate::ir::Type;
 use crate::isa::x64::args::AvxOpcode;
 use crate::isa::x64::inst::args::{
     AluRmiROpcode, Amode, Gpr, Imm8Reg, RegMem, RegMemImm, ShiftKind, SyntheticAmode,
@@ -247,7 +246,7 @@ pub(crate) fn check(
             undefined_result(ctx, vcode, dst, 64, 64)?;
             Ok(())
         }
-        Inst::MulHi {
+        Inst::Mul {
             size,
             dst_lo,
             dst_hi,
@@ -264,7 +263,17 @@ pub(crate) fn check(
             undefined_result(ctx, vcode, dst_hi, 64, 64)?;
             Ok(())
         }
-        Inst::UMulLo {
+        Inst::Mul8 { dst, ref src2, .. } => {
+            match <&RegMem>::from(src2) {
+                RegMem::Mem { ref addr } => {
+                    check_load(ctx, None, addr, vcode, I8, 64)?;
+                }
+                RegMem::Reg { .. } => {}
+            }
+            undefined_result(ctx, vcode, dst, 64, 64)?;
+            Ok(())
+        }
+        Inst::IMul {
             size,
             dst,
             ref src2,
@@ -279,7 +288,21 @@ pub(crate) fn check(
             undefined_result(ctx, vcode, dst, 64, 64)?;
             Ok(())
         }
-
+        Inst::IMulImm {
+            size,
+            dst,
+            ref src1,
+            ..
+        } => {
+            match <&RegMem>::from(src1) {
+                RegMem::Mem { ref addr } => {
+                    check_load(ctx, None, addr, vcode, size.to_type(), 64)?;
+                }
+                RegMem::Reg { .. } => {}
+            }
+            undefined_result(ctx, vcode, dst, 64, 64)?;
+            Ok(())
+        }
         Inst::CheckedSRemSeq {
             dst_quotient,
             dst_remainder,
