@@ -8,13 +8,12 @@
 use crate::environ::{FuncEnvironment, GlobalVariable, ModuleEnvironment, TargetEnvironment};
 use crate::func_translator::FuncTranslator;
 use crate::state::FuncTranslationState;
-use crate::WasmType;
+use crate::WasmValType;
 use crate::{
     DataIndex, DefinedFuncIndex, ElemIndex, FuncIndex, Global, GlobalIndex, GlobalInit, Heap,
     HeapData, HeapStyle, Memory, MemoryIndex, Table, TableIndex, TypeConvert, TypeIndex,
     WasmFuncType, WasmHeapType, WasmResult,
 };
-use core::convert::TryFrom;
 use cranelift_codegen::cursor::FuncCursor;
 use cranelift_codegen::ir::immediates::{Offset32, Uimm64};
 use cranelift_codegen::ir::{self, InstBuilder};
@@ -280,12 +279,12 @@ impl<'dummy_environment> FuncEnvironment for DummyFuncEnvironment<'dummy_environ
             gv: vmctx,
             offset,
             ty: match self.mod_info.globals[index].entity.wasm_ty {
-                WasmType::I32 => ir::types::I32,
-                WasmType::I64 => ir::types::I64,
-                WasmType::F32 => ir::types::F32,
-                WasmType::F64 => ir::types::F64,
-                WasmType::V128 => ir::types::I8X16,
-                WasmType::Ref(_) => ir::types::R64,
+                WasmValType::I32 => ir::types::I32,
+                WasmValType::I64 => ir::types::I64,
+                WasmValType::F32 => ir::types::F32,
+                WasmValType::F64 => ir::types::F64,
+                WasmValType::V128 => ir::types::I8X16,
+                WasmValType::Ref(_) => ir::types::R64,
             },
         })
     }
@@ -726,19 +725,19 @@ impl TargetEnvironment for DummyEnvironment {
 impl<'data> ModuleEnvironment<'data> for DummyEnvironment {
     fn declare_type_func(&mut self, wasm: WasmFuncType) -> WasmResult<()> {
         let mut sig = ir::Signature::new(CallConv::Fast);
-        let mut cvt = |ty: &WasmType| {
+        let mut cvt = |ty: &WasmValType| {
             let reference_type = match self.pointer_type() {
                 ir::types::I32 => ir::types::R32,
                 ir::types::I64 => ir::types::R64,
                 _ => panic!("unsupported pointer type"),
             };
             ir::AbiParam::new(match ty {
-                WasmType::I32 => ir::types::I32,
-                WasmType::I64 => ir::types::I64,
-                WasmType::F32 => ir::types::F32,
-                WasmType::F64 => ir::types::F64,
-                WasmType::V128 => ir::types::I8X16,
-                WasmType::Ref(_) => reference_type,
+                WasmValType::I32 => ir::types::I32,
+                WasmValType::I64 => ir::types::I64,
+                WasmValType::F32 => ir::types::F32,
+                WasmValType::F64 => ir::types::F64,
+                WasmValType::V128 => ir::types::I8X16,
+                WasmValType::Ref(_) => reference_type,
             })
         };
         sig.params.extend(wasm.params().iter().map(&mut cvt));
