@@ -45,12 +45,10 @@
 //! side-effectful initializers are emitted to the `GlobalInitializer` list in the
 //! final `Component`.
 
-use crate::component::translate::adapt::{Adapter, AdapterOptions};
 use crate::component::translate::*;
-use crate::{EntityType, PrimaryMap};
-use indexmap::IndexMap;
+use crate::EntityType;
 use std::borrow::Cow;
-use wasmparser::types::{ComponentAnyTypeId, ComponentEntityType, ComponentInstanceTypeId};
+use wasmparser::types::ComponentAnyTypeId;
 
 pub(super) fn run(
     types: &mut ComponentTypesBuilder,
@@ -330,7 +328,10 @@ enum ComponentInstanceDef<'a> {
     //
     // FIXME: same as the issue on `ComponentClosure` where this is cloned a lot
     // and may need `Rc`.
-    Items(IndexMap<&'a str, ComponentItemDef<'a>>),
+    Items(
+        IndexMap<&'a str, ComponentItemDef<'a>>,
+        Option<TypeComponentInstanceIndex>,
+    ),
 }
 
 #[derive(Clone)]
@@ -397,7 +398,7 @@ impl<'a> Inliner<'a> {
                     match frames.last_mut() {
                         Some((parent, _)) => {
                             parent.finish_instantiate(
-                                ComponentInstanceDef::Items(exports),
+                                ComponentInstanceDef::Items(exports, None),
                                 instance_ty.unwrap(),
                                 types,
                             );
@@ -783,14 +784,16 @@ impl<'a> Inliner<'a> {
                 return Ok(Some(frame));
             }
 
-            ComponentSynthetic(map) => {
+            ComponentSynthetic(map, ty) => {
                 let items = map
                     .iter()
                     .map(|(name, index)| Ok((*name, frame.item(*index, types)?)))
                     .collect::<Result<_>>()?;
+                let types_ref = frame.translation.types_ref();
+                let ty = types.convert_instance(types_ref, *ty)?;
                 frame
                     .component_instances
-                    .push(ComponentInstanceDef::Items(items));
+                    .push(ComponentInstanceDef::Items(items, Some(ty)));
             }
 
             // Core wasm aliases, this and the cases below, are creating
@@ -846,7 +849,7 @@ impl<'a> Inliner<'a> {
                     // through instantiation of a component or through a
                     // synthetic renaming of items we just schlep around the
                     // definitions of various items here.
-                    ComponentInstanceDef::Items(map) => frame.push_item(map[*name].clone()),
+                    ComponentInstanceDef::Items(map, _) => frame.push_item(map[*name].clone()),
                 }
             }
 
@@ -1029,7 +1032,10 @@ impl<'a> Inliner<'a> {
             // from.
             ComponentItemDef::Module(module) => match module {
                 ModuleDef::Static(idx) => dfg::Export::ModuleStatic(idx),
-                ModuleDef::Import(path, _) => dfg::Export::ModuleImport(self.runtime_import(&path)),
+                ModuleDef::Import(path, ty) => dfg::Export::ModuleImport {
+                    ty,
+                    import: self.runtime_import(&path),
+                },
             },
 
             ComponentItemDef::Func(func) => match func {
@@ -1051,7 +1057,7 @@ impl<'a> Inliner<'a> {
             },
 
             ComponentItemDef::Instance(instance) => {
-                let mut result = IndexMap::new();
+                let mut exports = IndexMap::new();
                 match instance {
                     // If this instance is one that was originally imported by
                     // the component itself then the imports are translated here
@@ -1064,20 +1070,24 @@ impl<'a> Inliner<'a> {
                         for (name, ty) in types[ty].exports.iter() {
                             let path = path.push(name);
                             let def = ComponentItemDef::from_import(path, *ty)?;
-                            self.record_export(name, def, types, &mut result)?;
+                            self.record_export(name, def, types, &mut exports)?;
+                        }
+                        dfg::Export::Instance {
+                            ty: Some(ty),
+                            exports,
                         }
                     }
 
                     // An exported instance which is itself a bag of items is
-                    // translated recursively here to our `result` map which is
+                    // translated recursively here to our `exports` map which is
                     // the bag of items we're exporting.
-                    ComponentInstanceDef::Items(map) => {
+                    ComponentInstanceDef::Items(map, ty) => {
                         for (name, def) in map {
-                            self.record_export(name, def, types, &mut result)?;
+                            self.record_export(name, def, types, &mut exports)?;
                         }
+                        dfg::Export::Instance { ty, exports }
                     }
                 }
-                dfg::Export::Instance(result)
             }
 
             // FIXME(#4283) should make an official decision on whether this is
@@ -1292,7 +1302,7 @@ impl<'a> ComponentItemDef<'a> {
             cur = match instance {
                 // If this instance is a "bag of things" then this is as easy as
                 // looking up the name in the bag of names.
-                ComponentInstanceDef::Items(names) => names[element].clone(),
+                ComponentInstanceDef::Items(names, _) => names[element].clone(),
 
                 // If, however, this instance is an imported instance then this
                 // is a further projection within the import with one more path
