@@ -1,17 +1,15 @@
 use crate::module::{
     FuncRefIndex, Initializer, MemoryInitialization, MemoryInitializer, MemoryPlan, Module,
-    ModuleType, TablePlan, TableSegment,
+    ModuleType, TableElementExpression, TablePlan, TableSegment, TableSegmentElements,
 };
 use crate::{
     DataIndex, DefinedFuncIndex, ElemIndex, EntityIndex, EntityType, FuncIndex, GlobalIndex,
-    GlobalInit, MemoryIndex, ModuleTypesBuilder, PrimaryMap, SignatureIndex, TableIndex,
-    TableInitialValue, Tunables, TypeConvert, TypeIndex, Unsigned, WasmError, WasmHeapType,
-    WasmResult, WasmType, WasmparserTypeConverter,
+    GlobalInit, MemoryIndex, ModuleTypesBuilder, PrimaryMap, TableIndex, TableInitialValue,
+    Tunables, TypeConvert, TypeIndex, Unsigned, WasmError, WasmHeapType, WasmResult, WasmValType,
+    WasmparserTypeConverter,
 };
-use cranelift_entity::packed_option::ReservedValue;
 use std::borrow::Cow;
 use std::collections::HashMap;
-use std::convert::TryFrom;
 use std::path::PathBuf;
 use std::sync::Arc;
 use wasmparser::types::{CoreTypeId, Types};
@@ -20,6 +18,7 @@ use wasmparser::{
     ExternalKind, FuncToValidate, FunctionBody, NameSectionReader, Naming, Operator, Parser,
     Payload, TypeRef, Validator, ValidatorResources,
 };
+use wasmtime_types::ModuleInternedTypeIndex;
 
 /// Object containing the standalone environment information.
 pub struct ModuleEnvironment<'a, 'data> {
@@ -55,7 +54,7 @@ pub struct ModuleTranslation<'data> {
     /// A list of type signatures which are considered exported from this
     /// module, or those that can possibly be called. This list is sorted, and
     /// trampolines for each of these signatures are required.
-    pub exported_signatures: Vec<SignatureIndex>,
+    pub exported_signatures: Vec<ModuleInternedTypeIndex>,
 
     /// DWARF debug information, if enabled, parsed from the module.
     pub debuginfo: DebugInfoData<'data>,
@@ -152,8 +151,8 @@ pub struct WasmFileInfo {
 #[derive(Debug)]
 #[allow(missing_docs)]
 pub struct FunctionMetadata {
-    pub params: Box<[WasmType]>,
-    pub locals: Box<[(u32, WasmType)]>,
+    pub params: Box<[WasmValType]>,
+    pub locals: Box<[(u32, WasmValType)]>,
 }
 
 impl<'a, 'data> ModuleEnvironment<'a, 'data> {
@@ -320,6 +319,10 @@ impl<'a, 'data> ModuleEnvironment<'a, 'data> {
                                     self.flag_func_escaped(index);
                                     TableInitialValue::FuncRef(index)
                                 }
+                                Operator::GlobalGet { global_index } => {
+                                    let index = GlobalIndex::from_u32(global_index);
+                                    TableInitialValue::GlobalGet(index)
+                                }
                                 s => {
                                     return Err(WasmError::Unsupported(format!(
                                         "unsupported init expr in table section: {:?}",
@@ -449,25 +452,31 @@ impl<'a, 'data> ModuleEnvironment<'a, 'data> {
                     // possible to create anything other than a `ref.null
                     // extern` for externref segments, so those just get
                     // translated to the reserved value of `FuncIndex`.
-                    let mut elements = Vec::new();
-                    match items {
+                    let elements = match items {
                         ElementItems::Functions(funcs) => {
-                            elements.reserve(usize::try_from(funcs.count()).unwrap());
+                            let mut elems =
+                                Vec::with_capacity(usize::try_from(funcs.count()).unwrap());
                             for func in funcs {
                                 let func = FuncIndex::from_u32(func?);
                                 self.flag_func_escaped(func);
-                                elements.push(func);
+                                elems.push(func);
                             }
+                            TableSegmentElements::Functions(elems.into())
                         }
-                        ElementItems::Expressions(_ty, funcs) => {
-                            elements.reserve(usize::try_from(funcs.count()).unwrap());
-                            for func in funcs {
-                                let func = match func?.get_binary_reader().read_operator()? {
-                                    Operator::RefNull { .. } => FuncIndex::reserved_value(),
+                        ElementItems::Expressions(_ty, items) => {
+                            let mut exprs =
+                                Vec::with_capacity(usize::try_from(items.count()).unwrap());
+                            for expr in items {
+                                let expr = match expr?.get_binary_reader().read_operator()? {
+                                    Operator::RefNull { .. } => TableElementExpression::Null,
                                     Operator::RefFunc { function_index } => {
                                         let func = FuncIndex::from_u32(function_index);
                                         self.flag_func_escaped(func);
-                                        func
+                                        TableElementExpression::Function(func)
+                                    }
+                                    Operator::GlobalGet { global_index } => {
+                                        let global = GlobalIndex::from_u32(global_index);
+                                        TableElementExpression::GlobalGet(global)
                                     }
                                     s => {
                                         return Err(WasmError::Unsupported(format!(
@@ -476,10 +485,11 @@ impl<'a, 'data> ModuleEnvironment<'a, 'data> {
                                         )));
                                     }
                                 };
-                                elements.push(func);
+                                exprs.push(expr);
                             }
+                            TableSegmentElements::Expressions(exprs.into())
                         }
-                    }
+                    };
 
                     match kind {
                         ElementKind::Active {
@@ -874,6 +884,7 @@ and for re-adding support for interface types you can see this issue:
                 | wasmparser::Name::Memory(_)
                 | wasmparser::Name::Element(_)
                 | wasmparser::Name::Data(_)
+                | wasmparser::Name::Tag(_)
                 | wasmparser::Name::Unknown { .. } => {}
             }
         }
