@@ -21,7 +21,7 @@ fn main() {
     let uri = res.header("x-wasmtime-test-uri").unwrap();
     assert_eq!(
         std::str::from_utf8(uri).unwrap(),
-        format!("http://{addr}/get?some=arg&goes=here")
+        format!("/get?some=arg&goes=here")
     );
     assert_eq!(res.body, b"");
 }
