@@ -12,5 +12,7 @@ fn main() {
     assert_eq!(res.status, 200);
     let method = res.header("x-wasmtime-test-method").unwrap();
     assert_eq!(std::str::from_utf8(method).unwrap(), "PUT");
+    let uri = res.header("x-wasmtime-test-uri").unwrap();
+    assert_eq!(std::str::from_utf8(uri).unwrap(), format!("/put"));
     assert_eq!(res.body, b"");
 }
