@@ -10,9 +10,29 @@ use std::fmt;
 mod error;
 pub use error::*;
 
-/// WebAssembly value type -- equivalent of `wasmparser`'s Type.
+/// A trait for things that can trace all type-to-type edges, aka all type
+/// indices within this thing.
+pub trait TypeTrace {
+    /// Visit each edge.
+    ///
+    /// The function can break out of tracing by returning `Err(E)`.
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>;
+
+    /// Visit each edge, mutably.
+    ///
+    /// Allows updating edges.
+    ///
+    /// The function can break out of tracing by returning `Err(E)`.
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>;
+}
+
+/// WebAssembly value type -- equivalent of `wasmparser::ValType`.
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
-pub enum WasmType {
+pub enum WasmValType {
     /// I32 type
     I32,
     /// I64 type
@@ -27,15 +47,45 @@ pub enum WasmType {
     Ref(WasmRefType),
 }
 
-impl fmt::Display for WasmType {
+impl fmt::Display for WasmValType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            WasmType::I32 => write!(f, "i32"),
-            WasmType::I64 => write!(f, "i64"),
-            WasmType::F32 => write!(f, "f32"),
-            WasmType::F64 => write!(f, "f64"),
-            WasmType::V128 => write!(f, "v128"),
-            WasmType::Ref(rt) => write!(f, "{rt}"),
+            WasmValType::I32 => write!(f, "i32"),
+            WasmValType::I64 => write!(f, "i64"),
+            WasmValType::F32 => write!(f, "f32"),
+            WasmValType::F64 => write!(f, "f64"),
+            WasmValType::V128 => write!(f, "v128"),
+            WasmValType::Ref(rt) => write!(f, "{rt}"),
+        }
+    }
+}
+
+impl TypeTrace for WasmValType {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        match self {
+            WasmValType::Ref(r) => r.trace(func),
+            WasmValType::I32
+            | WasmValType::I64
+            | WasmValType::F32
+            | WasmValType::F64
+            | WasmValType::V128 => Ok(()),
+        }
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        match self {
+            WasmValType::Ref(r) => r.trace_mut(func),
+            WasmValType::I32
+            | WasmValType::I64
+            | WasmValType::F32
+            | WasmValType::F64
+            | WasmValType::V128 => Ok(()),
         }
     }
 }
@@ -47,6 +97,22 @@ pub struct WasmRefType {
     pub heap_type: WasmHeapType,
 }
 
+impl TypeTrace for WasmRefType {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        self.heap_type.trace(func)
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        self.heap_type.trace_mut(func)
+    }
+}
+
 impl WasmRefType {
     pub const EXTERNREF: WasmRefType = WasmRefType {
         nullable: true,
@@ -74,30 +140,67 @@ impl fmt::Display for WasmRefType {
     }
 }
 
+/// An interned type index, either at the module or engine level.
+///
+/// Roughly equivalent to `wasmparser::UnpackedIndex`, although doesn't have to
+/// concern itself with recursion-group-local indices.
+#[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
+pub enum EngineOrModuleTypeIndex {
+    /// An index within a global namespace across all modules that can interact
+    /// with each other (in practice this is a `VMSharedTypeIndex` at the per
+    /// `wasmtime::Engine` level).
+    Engine(u32),
+    /// An index within the current Wasm module.
+    Module(ModuleInternedTypeIndex),
+}
+
+impl fmt::Display for EngineOrModuleTypeIndex {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        match self {
+            Self::Engine(i) => write!(f, "(engine {i})"),
+            Self::Module(i) => write!(f, "(module {})", i.as_u32()),
+        }
+    }
+}
+
 /// WebAssembly heap type -- equivalent of `wasmparser`'s HeapType
 #[derive(Debug, Copy, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum WasmHeapType {
-    Func,
     Extern,
-    // FIXME: the `SignatureIndex` payload here is not suitable given all the
-    // contexts that this type is used within. For example the Engine in
-    // wasmtime hashes this index which is not appropriate because the index is
-    // not globally unique.
-    //
-    // This probably needs to become `WasmHeapType<T>` where all of translation
-    // uses `WasmHeapType<SignatureIndex>` and all of engine-level "stuff"  uses
-    // `WasmHeapType<VMSharedSignatureIndex>`. This `<T>` would need to be
-    // propagated to quite a few locations though so it's left for a future
-    // refactoring at this time.
-    TypedFunc(SignatureIndex),
+    Func,
+    Concrete(EngineOrModuleTypeIndex),
+    NoFunc,
 }
 
 impl fmt::Display for WasmHeapType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
-            Self::Func => write!(f, "func"),
             Self::Extern => write!(f, "extern"),
-            Self::TypedFunc(i) => write!(f, "func_sig{}", i.as_u32()),
+            Self::Func => write!(f, "func"),
+            Self::Concrete(i) => write!(f, "{i}"),
+            Self::NoFunc => write!(f, "nofunc"),
+        }
+    }
+}
+
+impl TypeTrace for WasmHeapType {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        match *self {
+            Self::Concrete(i) => func(i),
+            Self::Func | Self::NoFunc | Self::Extern => Ok(()),
+        }
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        match self {
+            Self::Concrete(i) => func(i),
+            Self::Func | Self::NoFunc | Self::Extern => Ok(()),
         }
     }
 }
@@ -105,26 +208,54 @@ impl fmt::Display for WasmHeapType {
 /// WebAssembly function type -- equivalent of `wasmparser`'s FuncType.
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Serialize, Deserialize)]
 pub struct WasmFuncType {
-    params: Box<[WasmType]>,
+    params: Box<[WasmValType]>,
     externref_params_count: usize,
-    returns: Box<[WasmType]>,
+    returns: Box<[WasmValType]>,
     externref_returns_count: usize,
 }
 
+impl TypeTrace for WasmFuncType {
+    fn trace<F, E>(&self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        for p in self.params.iter() {
+            p.trace(func)?;
+        }
+        for r in self.returns.iter() {
+            r.trace(func)?;
+        }
+        Ok(())
+    }
+
+    fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
+    where
+        F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
+    {
+        for p in self.params.iter_mut() {
+            p.trace_mut(func)?;
+        }
+        for r in self.returns.iter_mut() {
+            r.trace_mut(func)?;
+        }
+        Ok(())
+    }
+}
+
 impl WasmFuncType {
     #[inline]
-    pub fn new(params: Box<[WasmType]>, returns: Box<[WasmType]>) -> Self {
+    pub fn new(params: Box<[WasmValType]>, returns: Box<[WasmValType]>) -> Self {
         let externref_params_count = params
             .iter()
             .filter(|p| match **p {
-                WasmType::Ref(rt) => rt.heap_type == WasmHeapType::Extern,
+                WasmValType::Ref(rt) => rt.heap_type == WasmHeapType::Extern,
                 _ => false,
             })
             .count();
         let externref_returns_count = returns
             .iter()
             .filter(|r| match **r {
-                WasmType::Ref(rt) => rt.heap_type == WasmHeapType::Extern,
+                WasmValType::Ref(rt) => rt.heap_type == WasmHeapType::Extern,
                 _ => false,
             })
             .count();
@@ -138,7 +269,7 @@ impl WasmFuncType {
 
     /// Function params types.
     #[inline]
-    pub fn params(&self) -> &[WasmType] {
+    pub fn params(&self) -> &[WasmValType] {
         &self.params
     }
 
@@ -150,7 +281,7 @@ impl WasmFuncType {
 
     /// Returns params types.
     #[inline]
-    pub fn returns(&self) -> &[WasmType] {
+    pub fn returns(&self) -> &[WasmValType] {
         &self.returns
     }
 
@@ -206,10 +337,19 @@ entity_impl!(GlobalIndex);
 pub struct MemoryIndex(u32);
 entity_impl!(MemoryIndex);
 
-/// Index type of a signature (imported or defined) inside the WebAssembly module.
+/// Index type of a type (imported or defined) inside the WebAssembly module.
 #[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
-pub struct SignatureIndex(u32);
-entity_impl!(SignatureIndex);
+pub struct TypeIndex(u32);
+entity_impl!(TypeIndex);
+
+/// Index type of a deduplicated type (imported or defined) inside a WebAssembly
+/// module.
+///
+/// Note that this is deduplicated only at the level of a WebAssembly module,
+/// not at the level of a whole store or engine.
+#[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
+pub struct ModuleInternedTypeIndex(u32);
+entity_impl!(ModuleInternedTypeIndex);
 
 /// Index type of a passive data segment inside the WebAssembly module.
 #[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
@@ -221,11 +361,6 @@ entity_impl!(DataIndex);
 pub struct ElemIndex(u32);
 entity_impl!(ElemIndex);
 
-/// Index type of a type inside the WebAssembly module.
-#[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
-pub struct TypeIndex(u32);
-entity_impl!(TypeIndex);
-
 /// Index type of an event inside the WebAssembly module.
 #[derive(Copy, Clone, PartialEq, Eq, Hash, PartialOrd, Ord, Debug, Serialize, Deserialize)]
 pub struct TagIndex(u32);
@@ -291,7 +426,7 @@ pub enum EntityType {
     Table(Table),
     /// A function type where the index points to the type section and records a
     /// function signature.
-    Function(SignatureIndex),
+    Function(ModuleInternedTypeIndex),
 }
 
 impl EntityType {
@@ -328,7 +463,7 @@ impl EntityType {
     }
 
     /// Assert that this entity is a function
-    pub fn unwrap_func(&self) -> SignatureIndex {
+    pub fn unwrap_func(&self) -> ModuleInternedTypeIndex {
         match self {
             EntityType::Function(g) => *g,
             _ => panic!("not a func"),
@@ -346,7 +481,7 @@ impl EntityType {
 #[derive(Debug, Clone, Copy, Hash, Eq, PartialEq, Serialize, Deserialize)]
 pub struct Global {
     /// The Wasm type of the value stored in the global.
-    pub wasm_ty: crate::WasmType,
+    pub wasm_ty: crate::WasmValType,
     /// A flag indicating whether the value may change at runtime.
     pub mutability: bool,
 }
@@ -459,14 +594,14 @@ pub trait TypeConvert {
     }
 
     /// Converts a wasmparser value type to a wasmtime type
-    fn convert_valtype(&self, ty: wasmparser::ValType) -> WasmType {
+    fn convert_valtype(&self, ty: wasmparser::ValType) -> WasmValType {
         match ty {
-            wasmparser::ValType::I32 => WasmType::I32,
-            wasmparser::ValType::I64 => WasmType::I64,
-            wasmparser::ValType::F32 => WasmType::F32,
-            wasmparser::ValType::F64 => WasmType::F64,
-            wasmparser::ValType::V128 => WasmType::V128,
-            wasmparser::ValType::Ref(t) => WasmType::Ref(self.convert_ref_type(t)),
+            wasmparser::ValType::I32 => WasmValType::I32,
+            wasmparser::ValType::I64 => WasmValType::I64,
+            wasmparser::ValType::F32 => WasmValType::F32,
+            wasmparser::ValType::F64 => WasmValType::F64,
+            wasmparser::ValType::V128 => WasmValType::V128,
+            wasmparser::ValType::Ref(t) => WasmValType::Ref(self.convert_ref_type(t)),
         }
     }
 
@@ -481,14 +616,15 @@ pub trait TypeConvert {
     /// Converts a wasmparser heap type to a wasmtime type
     fn convert_heap_type(&self, ty: wasmparser::HeapType) -> WasmHeapType {
         match ty {
-            wasmparser::HeapType::Func => WasmHeapType::Func,
             wasmparser::HeapType::Extern => WasmHeapType::Extern,
+            wasmparser::HeapType::Func => WasmHeapType::Func,
+            wasmparser::HeapType::NoFunc => WasmHeapType::NoFunc,
             wasmparser::HeapType::Concrete(i) => self.lookup_heap_type(i),
 
             wasmparser::HeapType::Any
+            | wasmparser::HeapType::Exn
             | wasmparser::HeapType::None
             | wasmparser::HeapType::NoExtern
-            | wasmparser::HeapType::NoFunc
             | wasmparser::HeapType::Eq
             | wasmparser::HeapType::Struct
             | wasmparser::HeapType::Array
