@@ -46,6 +46,7 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     // wasmtime
     "wasmtime-asm-macros",
     "wasmtime-versioned-export-macros",
+    "wasmtime-slab",
     "wasmtime-component-util",
     "wasmtime-wit-bindgen",
     "wasmtime-component-macro",
@@ -56,7 +57,6 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     "wasmtime-runtime",
     "wasmtime-cranelift-shared",
     "wasmtime-cranelift",
-    "wasmtime-jit",
     "wasmtime-cache",
     "winch-codegen",
     "wasmtime-winch",
@@ -64,14 +64,14 @@ const CRATES_TO_PUBLISH: &[&str] = &[
     // wasi-common/wiggle
     "wiggle",
     "wasi-common",
-    "wasi-cap-std-sync",
-    "wasi-tokio",
     // other misc wasmtime crates
     "wasmtime-wasi",
     "wasmtime-wasi-http",
     "wasmtime-wasi-nn",
     "wasmtime-wasi-threads",
     "wasmtime-wast",
+    "wasmtime-c-api-macros",
+    "wasmtime-c-api-impl",
     "wasmtime-cli-flags",
     "wasmtime-explorer",
     "wasmtime-cli",
@@ -82,9 +82,6 @@ const CRATES_TO_PUBLISH: &[&str] = &[
 // releases since everything not mentioned here is just an organizational detail
 // that no one else should rely on.
 const PUBLIC_CRATES: &[&str] = &[
-    // just here to appease the script because these are submodules of this
-    // repository.
-    "witx",
     // these are actually public crates which we cannot break the API of in
     // patch releases.
     "wasmtime",
@@ -253,9 +250,6 @@ fn read_crate(ws: Option<&Workspace>, manifest: &Path) -> Crate {
     }
     let name = name.unwrap();
     let version = version.unwrap();
-    if ["witx", "witx-cli"].contains(&&name[..]) {
-        publish = false;
-    }
     Crate {
         manifest: manifest.to_path_buf(),
         name,
@@ -491,14 +485,6 @@ fn verify(crates: &[Crate]) {
     fs::create_dir_all(".cargo").unwrap();
     fs::write(".cargo/config.toml", vendor.stdout).unwrap();
 
-    // Vendor witx which wasn't vendored because it's a path dependency, but
-    // it'll need to be in our directory registry for crates that depend on it.
-    let witx = crates
-        .iter()
-        .find(|c| c.name == "witx" && c.manifest.iter().any(|p| p == "wasi-common"))
-        .unwrap();
-    verify_and_vendor(&witx);
-
     for krate in crates {
         if !krate.publish {
             continue;
@@ -512,7 +498,7 @@ fn verify(crates: &[Crate]) {
             .arg("--manifest-path")
             .arg(&krate.manifest)
             .env("CARGO_TARGET_DIR", "./target");
-        if krate.name == "witx" || krate.name.contains("wasi-nn") {
+        if krate.name.contains("wasi-nn") {
             cmd.arg("--no-verify");
         }
         let status = cmd.status().unwrap();
